#include <math.h>

typedef struct
{
	float x, y, z, thx, thy, thz, cx, cy, cz, sx, sy, sz;
} Camera;

typedef struct
{
	float x, y;
	int ix, iy;
	int beyond;
} Point2D;

typedef struct
{
	float x, y, z;
} Point3D;

void ChangeCameraThetaX(Camera *C, float thx, int set)
{
	C->thx = fmod(set ? thx : C->thx + thx, 2.0 * M_PI);
	C->cx = cosf(C->thx);
	C->sx = sinf(C->thx);
}

void ChangeCameraThetaY(Camera *C, float thy, int set)
{
	C->thy = fmod(set ? thy : C->thy + thy, 2.0 * M_PI);
	C->cy = cosf(C->thy);
	C->sy = sinf(C->thy);
}

void ChangeCameraThetaZ(Camera *C, float thz, int set)
{
	C->thz = fmod(set ? thz : C->thz + thz, 2.0 * M_PI);
	C->cz = cosf(C->thz);
	C->sz = sinf(C->thz);
}

void CameraMove(Camera *C, float x, float y, float z)
{
	C->x += x * (C->cy * C->cz) + y * (C->sx * C->sy * C->cz * -C->cx * C->sz) + z * (C->cx * C->sy * C->cz + C->sx * C->sz);
	C->y += x * (C->cy * C->sz) + y * (C->sx * C->sy * C->sz + C->cx * C->cz) + z * (C->cx * C->sy * C->sz - C->sx * C->cz);
	C->z += x * -C->sy + y * (C->sx * C->cy) + z * (C->cx * C->cy);
}

void TransformPoint3DToPoint2D(Point3D *p3D, Camera *C, Point2D *p2D, int w, int h)
{
	float x, y, z, dx, dy, dz, ezpdz;

	if ((p3D == NULL) || (C == NULL) || (p2D == NULL))
	{
		return;
	}

	x = p3D->x - C->x;
	y = p3D->y - C->y;
	z = p3D->z - C->z;

	dz = C->cx * (C->cy * z + C->sy * (C->sz * y + C->cz * x)) - C->sx * (C->cz * y - C->sz * x);
	p2D->beyond = dz < 0.0;
	ezpdz = 1.0 / dz;

	dx = C->cy * (C->sz * y + C->cz * x) - C->sy * z;
	p2D->x = ezpdz * dx;
	p2D->ix = (int)round(((h / 2.0 ) * p2D->x) + (w / 2.0));
	if (p2D->ix < 0)
	{
		p2D->ix = 0;
	}
	else if (p2D->ix >= w)
	{
		p2D->ix = w - 1;
	}

	dy = C->sx * (C->cy * z + C->sy * (C->sz * y + C->cz * x)) + C->cx * (C->cz * y - C->sz * x);
	p2D->y = ezpdz * dy;
	p2D->iy = (int)round(((h / 2.0) * p2D->y) + (h / 2.0));
	if (p2D->iy < 0)
	{
		p2D->iy = 0;
	}
	else if (p2D->iy >= h)
	{
		p2D->iy = h - 1;
	}
}
