#ifndef _VIDEO_SDL1_C
#define _VIDEO_SDL1_C 1

#include <stdint.h>
#include <stdbool.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#define VIDEO_SDL1_ERROR_CANNOT_INIT_SUBSYSTEM	0x1
#define VIDEO_SLD1_ERROR_CANNOT_SET_GL_ATTRIB	0x2
#define VIDEO_SDL1_ERROR_CANNOT_OPEN_WINDOW	0x3

SDL_Surface *main_window;
bool sdl_upd;
uint16_t sdl_upd_x, sdl_upd_y, sdl_upd_w, sdl_upd_h;

int open_main_window(int w, int h)
{
	if (SDL_InitSubSystem(SDL_INIT_VIDEO) != 0)
	{
		return VIDEO_SDL1_ERROR_CANNOT_INIT_SUBSYSTEM;
	}
	if
	(
		(
			SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8) |
			SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8) |
			SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8) |
			SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 0) |
			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24) |
			SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1)
		) != 0
	)
	{
		SDL_Quit();
		return VIDEO_SLD1_ERROR_CANNOT_SET_GL_ATTRIB;
	}
	main_window = SDL_SetVideoMode(w, h, 32, SDL_OPENGL | SDL_OPENGLBLIT);
	if (main_window == NULL)
	{
		SDL_Quit();
		return VIDEO_SDL1_ERROR_CANNOT_OPEN_WINDOW;
	}
	glShadeModel(GL_FLAT);
	glCullFace(GL_BACK);
	glFrontFace(GL_CCW);
	glEnable(GL_CULL_FACE);
	glClearColor(0, 0, 0, 0);
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (float)w / (float)h, 1.0, 1024.0);
	SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
	return 0;
}

#define close_main_window() SDL_FreeSurface(main_window)
#define update_area(x, y, w, h) SDL_UpdateRect(main_window, (x), (y), (w), (h))
#define swap_buffers() SDL_GL_SwapBuffers()

pixel *calc_first_pixel_address_of_area(int x, int y)
{
	pixel *dst;

	dst = (pixel *)main_window->pixels;
	dst += (y * main_window->w) + x;
	return dst;
}

#define calc_modulo_for_area(width) (main_window->w - (width))

void color_area(pixel color, int x, int y, int w, int h)
{
	int cx, cy, modulo;
	pixel *dst;

	modulo = calc_modulo_for_area(w);
	dst = calc_first_pixel_address_of_area(x, y);
	for (cy = 0; cy < h; ++cy)
	{
		for (cx = 0; cx < w; ++cx)
		{
			*dst++ = color;
		}
		dst += modulo;
	}
	update_area(x, y, w, h);
}

#endif
